<?php

namespace Drupal\alt_text_validation\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form handler for the Text Rule add and edit forms.
 */
class AltTextRuleForm extends EntityForm {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state): array {
    $form = parent::form($form, $form_state);

    /** @var \Drupal\alt_text_validation\AltTextRuleInterface $entity */
    $entity = $this->entity;

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#default_value' => $entity->get('label'),
      '#required' => TRUE,
      '#description' => $this->t('The text rule label.'),
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $entity->get('id'),
      '#machine_name' => [
        'exists' => [$this, 'exists'],
        'source' => ['label'],
      ],
      '#disabled' => !$entity->isNew(),
    ];

    $form['rule_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Rule type'),
      '#default_value' => $entity->getRuleType() ?? '',
      '#empty_option' => $this->t('- select -'),
      '#empty_value' => '',
      '#options' => [
        'alt_is_filename' => $this->t('Cannot match image filename'),
        'alt_is_title' => $this->t('Cannot match image title'),
        'not_empty' => $this->t('Cannot be empty unless decorative'),
        'length_limit' => $this->t('Cannot exceed length'),
        'not_begin_with' => $this->t('Cannot begin with'),
        'not_equal' => $this->t('Cannot equal'),
        'not_contain' => $this->t('Cannot contain'),
        'not_end_with' => $this->t('Cannot end with'),
        'regex_match' => $this->t('Cannot match regex'),
      ],
      '#required' => TRUE,
    ];

    $form['content_char_limit'] = [
      '#type' => 'number',
      '#title' => $this->t('Content character limit'),
      '#default_value' => $entity->get('content_char_limit'),
      '#description' => $this->t('The number of characters allowed.'),
      '#min' => 0,
      '#states' => [
        'visible' => [
          ':input[name="rule_type"]' => ['value' => 'length_limit'],
        ],
        'required' => [
          ':input[name="rule_type"]' => ['value' => 'length_limit'],
        ],
      ],
    ];

    $form['text_to_compare'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Text to compare'),
      '#default_value' => $entity->get('text_to_compare'),
      '#description' => $this->t('Text string to match against..'),
      '#states' => [
        'visible' => [
          ':input[name="rule_type"]' => [
            ['value' => 'not_begin_with'],
            ['value' => 'not_equal'],
            ['value' => 'not_contain'],
            ['value' => 'not_end_with'],
            ['value' => 'regex_match'],
          ],
        ],
        'required' => [
          ':input[name="rule_type"]' => [
            ['value' => 'not_begin_with'],
            ['value' => 'not_equal'],
            ['value' => 'not_contain'],
            ['value' => 'not_end_with'],
            ['value' => 'regex_match'],
          ],
        ],
      ],
    ];

    $form['violation_action'] = [
      '#type' => 'select',
      '#title' => $this->t('Violation Action'),
      '#default_value' => $entity->get('violation_action') ?? '',
      '#empty_option' => $this->t('- select -'),
      '#empty_value' => '',
      '#options' => [
        'off' => $this->t('Off'),
        'warn' => $this->t('Warn'),
        'prevent' => $this->t('Prevent'),
      ],
      '#required' => TRUE,
    ];

    $form['violation_message'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Violation Message'),
      '#default_value' => $entity->get('violation_message') ?? '',
      '#description' => $this->t('Message to display when rule has been violated.'),
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state): void {
    $alt_text_rule = $this->entity;
    $status = $alt_text_rule->save();

    if ($status) {
      $this->messenger()->addMessage($this->t('Saved the %label Text Rule.', [
        '%label' => $alt_text_rule->label(),
      ]));
    }
    else {
      $this->messenger()->addMessage($this->t('Failed to save the %label Text Rule.', [
        '%label' => $alt_text_rule->label(),
      ]), 'error');
    }

    $form_state->setRedirect('entity.alt_text_rule.collection');
  }

  /**
   * Helper function to check whether a Text Rule config entity exists.
   *
   * @returns bool
   */
  public function exists($id): bool {
    $entity = $this->entityTypeManager
      ->getStorage('alt_text_rule')
      ->getQuery()
      ->accessCheck(FALSE)
      ->condition('id', $id)
      ->execute();
    return (bool) $entity;
  }

  /**
   * Constructs a TextRule config entity.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   Entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager) {
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static($container->get('entity_type.manager'));
  }

}
